@@ -32,17 +32,20 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.12'
+__version__ = '0.14'
 
 
 from config import Config
+import core
 from core import Core
+import core_helpers
 from core_helpers import CoreHelpers
 from errors import CoreError
 import embedded_files
 
 
-__all__ = ['CoreError', 'Core', 'CoreHelpers', 'Config', 'embedded_files']
+__all__ = ['CoreError', 'core', 'Core', 'core_helpers', 'CoreHelpers',
+           'Config', 'embedded_files']
 
 
 # vi: set ai aw sw=2:
