@@ -32,7 +32,7 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.12'
+__version__ = '0.14'
 
 
 import ConfigParser
@@ -77,9 +77,12 @@ class Config(object):
                               'run_as_username': 'str',
                               'server_killswitch': 'boolean',
                               'lock_file': 'str', 'ssl_key_file': 'str',
-                              'ssl_cert_file': 'str'},
+                              'server_log_file': 'str', 'ssl_cert_file': 'str'},
                    'credentials': {'authentication_method': 'str',
-                                   'exp_time': 'int'}}
+                                   'exp_time': 'int'},
+                   'exporter': {'backup_dir': 'str',
+                                'root_config_dir': 'str',
+                                'named_dir': 'str'}}
 
     for section in file_schema:
       self.config_file[section] = {}
