@@ -32,7 +32,7 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.12'
+__version__ = '0.14'
 
 
 # This is the default TTL for all records inserted into the database.
@@ -55,10 +55,12 @@ ACCESS_LEVELS = {'dns_admin': 128,
                  'user': 32,
                  'noop': 0}
 
-
 # Valid access rights for forward or reverse perms in the database.
 ACCESS_RIGHTS = ['rw', 'r']
 
+# This is a list of tables that are not audit logged when changes are made.
+# it is important not to overwrite these tables when doing a partial replay
+TABLES_NOT_AUDIT_LOGGED = ['audit_log', 'locks']
 
 # The SUPPORTED_METHODS hash contains a hash for every supported method.
 # 'check' indicates whether the target zone/IP range should be checked.
@@ -367,7 +369,32 @@ SUPPORTED_METHODS = {
     'ListRecordArgumentDefinitions':
                     {'check': False,
                      'write': True,
-                     'access_level': ACCESS_LEVELS['dns_admin']}}
+                     'access_level': ACCESS_LEVELS['dns_admin']},
+    'ListAuditLog':
+                    {'check': False,
+                     'write': False,
+                     'access_level': ACCESS_LEVELS['dns_admin']},
+    'ListCredentials':
+                    {'check': False,
+                     'write': False,
+                     'access_level': ACCESS_LEVELS['dns_admin']},
+    'MakeInfiniteCredential':
+                    {'check': False,
+                     'write': True,
+                     'access_level': ACCESS_LEVELS['dns_admin']},
+    'RemoveCredential':
+                    {'check': False,
+                     'write': True,
+                     'access_level': ACCESS_LEVELS['dns_admin']},
+    'SetMaintenanceFlag':
+                    {'check': False,
+                     'write': True,
+                     'access_level': ACCESS_LEVELS['dns_admin']},
+    'CheckMaintenanceFlag':
+                    {'check': False,
+                     'write': False,
+                     'access_level': ACCESS_LEVELS['user']}}
+
 
 
 # These are a subset of tables in the dtabase. They are enumerated here to
@@ -478,10 +505,13 @@ TABLES = {
                          {'reserved_word': 'ReservedWord'},
 
     'audit_log':
-                         {'audit_log_user_name': 'UnicodeString',
+                         {'audit_log_id': 'UnsignedInt',
+                          'audit_log_user_name': 'UnicodeString',
                           'action': 'UnicodeString',
-                          'data': 'ReservedWord',
+                          'data': 'PickleString',
                           'success': 'IntBool',
-                          'audit_log_timestamp': 'DateTime'}}
+                          'audit_log_timestamp': 'DateTime'},
+    'locks':
+        {'lock_name': 'UnicodeString', 'locked': 'IntBool'}}
 
 # vi: set ai aw sw=2:
