@@ -32,16 +32,23 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.12'
+__version__ = '0.14'
 
 
-import datetime
-import IPy
 import constants
 import core
 import errors
+import helpers_lib
+import user
+
+import datetime
 import math
 
+import IPy
+
+class RecordsBatchError(errors.CoreError):
+  pass
+
 class CoreHelpers(object):
   """Library of helper functions that extend the core functions."""
   def __init__(self, core_instance):
@@ -55,92 +62,22 @@ class CoreHelpers(object):
     self.user_instance = core_instance.user_instance
     self.log_instance = core_instance.log_instance
 
+  ### These functions just expose helpers_lib functions for the 
+  ### XML-RPC server. For doc strings see helpers_lib
   def ListAccessRights(self):
-    """Lists access rights.
-
-    Output:
-      list: list of access rights. ex: ['rw', 'r']
-    """
-    return constants.ACCESS_RIGHTS
+    return helpers_lib.ListAccessRights()
 
   def ReverseIP(self, ip_address):
-    """Reverse an IP address
-
-    Inputs:
-      ip_address: either an ipv4 or ipv6 string
-
-    Outputs:
-      string: reverse ip address
-    """
-    ip_object = IPy.IP(ip_address)
-    reverse_ip_string = ip_object.reverseName()
-    if( ip_object.version() == 4 ):
-      ip_parts = reverse_ip_string.split('.')
-      if( '-' in ip_parts[0] ):
-        range = ip_parts.pop(0).split('-')
-        num_ips = int(range[1]) - int(range[0]) + 1
-        netmask = int(32 - (math.log(num_ips) / math.log(2)))
-        last_octet = ip_parts.pop(0)
-        reverse_ip_string = '.'.join(ip_parts)
-        reverse_ip_string = '%s/%s.%s' % (last_octet, netmask,
-                                          reverse_ip_string)
-    return unicode(reverse_ip_string)
+    return helpers_lib.ReverseIP(ip_address)
 
   def UnReverseIP(self, ip_address):
-    """Un-Reverses reversed IP addresses
-
-    Inputs:
-      ip_address: either an ipv4 or ipv6 string (reversed)
-
-    Outputs:
-      string: forward ip address
-    """
-    mask = 0
-    cidr_parts = ip_address.split('/')
-    if( len(cidr_parts) == 2 ):
-      mask = cidr_parts[1].split('.')[0]
-      ip_address = ip_address.replace('/%s' % mask, '')
-    if( ip_address.endswith('in-addr.arpa.') ):
-      octets = 0
-      ip_array = ip_address.split('.')
-      ip_parts = []
-      while len(ip_array):
-        ip_part = ip_array.pop()
-        if( ip_part.isdigit() ):
-          ip_parts.append(ip_part)
-          octets += 1
-      new_ip = '.'.join(ip_parts)
-      if( mask ):
-        new_ip = '%s/%s' % (new_ip, mask)
-      elif( octets < 4 ):
-        new_ip = '%s/%s' % (new_ip, octets * 8)
-    elif( ip_address.endswith('ip6.arpa.') ):
-      ip_array = ip_address.split('.')[:-3]
-      ip_parts = []
-      while len(ip_array):
-        ip_parts.append('%s%s%s%s' % (ip_array.pop(), ip_array.pop(),
-                                      ip_array.pop(), ip_array.pop()))
-      new_ip = ':'.join(ip_parts)
-    else:
-      new_ip = ip_address
-
-    return new_ip
+    return helpers_lib.UnReverseIP(ip_address)
 
   def CIDRExpand(self, cidr_block):
-    """Expands a cidr block to a list of ip addreses
+    return helpers_lib.CIDRExpand(cidr_block)
 
-    Inputs:
-      cidr_block: string of cidr_block
-
-    Outputs:
-      list: list of ip addresses in strings
-    """
-    cidr_block = IPy.IP(cidr_block)
-    ip_address_list = []
-    for ip_address in cidr_block:
-      ip_address_list.append(unicode(ip_address.strFullsize()))
-
-    return ip_address_list
+  def ExpandIPV6(self, ip_address):
+    return helpers_lib.ExpandIPV6(ip_address)
 
   def ListLatestNamedConfig(self, dns_server_set):
     """Lists the latest named config string given dns server set
@@ -201,20 +138,6 @@ class CoreHelpers(object):
     self.core_instance.MakeRecord(u'aaaa', target, zone_name, record_args_dict,
                                   view_name, ttl)
 
-  def ExpandIPV6(self, ip_address):
-    """Expands a shorthand ipv6 address to a full ipv6 address
-
-    Inputs:
-      ip_address: string of ipv6 address
-
-    Outputs:
-      string: string of long ipv6 address
-    """
-    ipv6_address = IPy.IP(ip_address)
-    if( ipv6_address.version() != 6 ):
-      raise errors.CoreError('"%s" is not a valid IPV6 address.' % ipv6_address)
-    return ipv6_address.strFullsize()
-
   def GetPTRTarget(self, long_target, view_name=u'any'):
     """Gets the short PTR target given the long PTR target
 
@@ -282,7 +205,7 @@ class CoreHelpers(object):
     self.core_instance.RemoveRecord(u'ptr', target, zone_name,
                                     record_args_dict, view_name, ttl)
 
-  def ListRecordsByCIDRBlock(self, cidr_block, view_name=None):
+  def ListRecordsByCIDRBlock(self, cidr_block, view_name=None, zone_name=None):
     """Lists records in user given cidr block.
 
     Inputs:
@@ -316,12 +239,13 @@ class CoreHelpers(object):
 
     fwd_dict = self.db_instance.GetEmptyRowDict('records')
     fwd_dict['record_type'] = record_type
-    if( not view_name ):
-      view_name = u'any'
-    elif( view_name.endswith('_dep') or view_name == u'any' ):
+    if( view_name is not None and view_name.endswith('_dep')
+        or view_name == u'any' ):
       fwd_dict['record_view_dependency'] = view_name
-    else:
+      ptr_dict['record_view_dependency'] = view_name
+    elif( view_name is not None ):
       fwd_dict['record_view_dependency'] = '%s_dep' % view_name
+      ptr_dict['record_view_dependency'] = '%s_dep' % view_name
     fwd_args_dict = self.db_instance.GetEmptyRowDict(
         'record_arguments_records_assignments')
     fwd_args_dict['record_arguments_records_assignments_argument_name'] = (
@@ -360,16 +284,22 @@ class CoreHelpers(object):
     finally:
       self.db_instance.EndTransaction()
     records_dict = {}
+    orig_view = view_name
     for record in ptr_record_list:
-      zone_name = record['record_zone_name']
+      view_name = orig_view
+      record_zone_name = record['record_zone_name']
       db_view_name = record['record_view_dependency'].rsplit('_dep', 1)[0]
-      if( db_view_name != view_name and view_name != 'any'):
+      if( db_view_name != view_name and (db_view_name != u'any' and view_name is not None) ):
         continue
-      if( db_view_name not in zones[zone_name]  ):
+      if( zone_name and record_zone_name != zone_name ):
+        continue
+      if( view_name is None ):
+        view_name = db_view_name
+      if( db_view_name not in zones[record_zone_name]  ):
         raise errors.CoreError('No zone view combination found for '
                                '"%s" zone and "%s" view.' % (
-                                   zone_name, db_view_name))
-      zone_origin = zones[zone_name][db_view_name]['zone_origin']
+                                   record_zone_name, db_view_name))
+      zone_origin = zones[record_zone_name][db_view_name]['zone_origin']
       reverse_ip_address = '%s.%s' % (record['record_target'], zone_origin)
       ip_address = self.UnReverseIP(reverse_ip_address)
       if( IPy.IP(ip_address) in user_cidr ):
@@ -381,18 +311,23 @@ class CoreHelpers(object):
             u'forward': False, u'host': record['argument_value'].rstrip('.'),
             u'zone': record['record_zone_name'], 'zone_origin': zone_origin})
     for record in fwd_record_list:
+      view_name = orig_view
       ip_address = record['argument_value']
-      zone_name = record['record_zone_name']
+      record_zone_name = record['record_zone_name']
       db_view_name = record['record_view_dependency'].rsplit('_dep', 1)[0]
-      if( db_view_name != view_name and view_name != 'any'):
+      if( db_view_name != view_name and (db_view_name != u'any' and view_name is not None) ):
         continue
-      zone_origin = zones[zone_name][db_view_name]['zone_origin']
+      if( zone_name and record_zone_name != zone_name ):
+        continue
+      if( view_name is None ):
+        view_name = db_view_name
+      zone_origin = zones[record_zone_name][db_view_name]['zone_origin']
       if( IPy.IP(ip_address) in user_cidr ):
-        if( not view_name in records_dict ):
-          records_dict[view_name] = {}
-        if( not ip_address in records_dict[view_name] ):
-           records_dict[view_name][ip_address] = []
-        records_dict[view_name][ip_address].append({
+        if( not db_view_name in records_dict ):
+          records_dict[db_view_name] = {}
+        if( not ip_address in records_dict[db_view_name] ):
+           records_dict[db_view_name][ip_address] = []
+        records_dict[db_view_name][ip_address].append({
             u'forward': True, u'host': '%s.%s' % (
                 record['record_target'], zone_origin.rstrip('.')),
             u'zone': record['record_zone_name'],
@@ -412,9 +347,6 @@ class CoreHelpers(object):
     Outputs:
       list: list of dictionarires from ListNamedConfGlobalOptions
     """
-    if( timestamp is not None ):
-      timestamp = datetime.datetime.strptime(timestamp.value,
-                                             "%Y%m%dT%H:%M:%S")
     return self.core_instance.ListNamedConfGlobalOptions(
         option_id, dns_server_set, timestamp)
 
@@ -449,6 +381,8 @@ class CoreHelpers(object):
     Outputs:
       int: number of rows modified
     """
+    function_name, current_args = helpers_lib.GetFunctionNameAndArgs()
+
     record_arguments_record_assignments_dict = (
         self.db_instance.GetEmptyRowDict(
             'record_arguments_records_assignments'))
@@ -482,8 +416,13 @@ class CoreHelpers(object):
             raise errors.CoreError('Incorrect number of records found!')
           try:
             self.core_instance.user_instance.Authorize(
-                'RemoveRecord', target=found_records_dict[0]['record_target'])
-          except self.core_instance.user_instance.AuthError:
+                'RemoveRecord',
+                 record_data=
+                     {'target': records_dict['record_target'],
+                      'zone_name': records_dict['record_zone_name'],
+                      'view_name': records_dict['record_view_dependency']},
+                current_transaction=True)
+          except user.AuthError:
             continue
           row_count += self.db_instance.RemoveRow(
               'records', found_records_dict[0])
@@ -505,12 +444,8 @@ class CoreHelpers(object):
       if( log_list ):
         remove_record_string = ' '.join(log_list)
     finally:
-      self.core_instance.log_instance.LogAction(
-          self.core_instance.user_instance.user_name,
-          u'RemoveCNamesByAssignmentHost',
-          u'hostname: %s cnames_removed: (%s)' % (
-              hostname, remove_record_string), success)
-
+      self.log_instance.LogAction(self.user_instance.user_name, function_name,
+                                  current_args, success)
     return row_count
 
   def ProcessRecordsBatch(self, delete_records=[], add_records=[]):
@@ -526,6 +461,8 @@ class CoreHelpers(object):
     Outputs:
       int: row count
     """
+    function_name, current_args = helpers_lib.GetFunctionNameAndArgs()
+
     log_dict = {'delete': [], 'add': []}
     row_count = 0
     changed_view_dep = []
@@ -535,8 +472,6 @@ class CoreHelpers(object):
       try:
         # REMOVE RECORDS
         for record in delete_records:
-          self.user_instance.Authorize('RemoveRecord', target=record[
-              'record_target'])
           records_dict = self.db_instance.GetEmptyRowDict('records')
           records_dict['record_type'] = record['record_type']
           records_dict['record_target'] = record['record_target']
@@ -547,12 +482,20 @@ class CoreHelpers(object):
             view_name = '%s_dep' % record['view_name']
           changed_view_dep.append((view_name, record['record_zone_name']))
           records_dict['record_view_dependency'] = view_name
+          self.user_instance.Authorize('RemoveRecord',
+              record_data =
+                  {'target': record['record_target'],
+                   'zone_name': record['record_zone_name'],
+                   'view_name': records_dict['record_view_dependency']},
+              current_transaction=True)
+
           if( 'record_ttl' in record ):
             records_dict['record_ttl'] = record['record_ttl']
           args_list = []
           for argument in record['record_arguments']:
             if( record['record_arguments'][argument] is None ):
-              raise errors.CoreError('"%s" cannot be None' % argument)
+              raise errors.CoreErrore('%s: "%s" cannot be None' % (
+                  record['record_target'], argument))
             args_list.append(
                 {u'record_arguments_records_assignments_argument_name':
                      argument,
@@ -586,19 +529,23 @@ class CoreHelpers(object):
             new_records = self.db_instance.ListRow('records', records_dict)
             rows_deleted = self.db_instance.RemoveRow('records', new_records[0])
             if( not rows_deleted ):
-              raise core.RecordError('Record with record_id %s not found' %
-                                     final_id)
+              raise RecordsBatchError(
+                  '%s: Record not found' % record['record_target'])
             log_dict['delete'].append(record)
             row_count += 1
 
         # ADD RECORDS
         for record in add_records:
-          self.user_instance.Authorize('MakeRecord', target=record[
-              'record_target'])
           view_name = record['view_name']
           if( not record['view_name'].endswith('_dep') and record[
                 'view_name'] != u'any'):
             view_name = '%s_dep' % record['view_name']
+          self.user_instance.Authorize('MakeRecord', 
+              record_data = {
+                  'target': record['record_target'],
+                  'zone_name': record['record_zone_name'],
+                  'view_name': view_name},
+              current_transaction=True)
           if( record['record_type'] == u'ptr' ):
             if( record['record_arguments'][
                 'assignment_host'].startswith('@.') ):
@@ -614,29 +561,55 @@ class CoreHelpers(object):
           records_dict = {'records_id': None,
                           'record_target': record['record_target'],
                           'record_type': None,
-                          'record_ttl': ttl,
+                          'record_ttl': None,
                           'record_zone_name': record['record_zone_name'],
                           'record_view_dependency': view_name,
-                          'record_last_user': self.user_instance.GetUserName()}
-          if( record['record_type'] == 'a' or record['record_type'] == 'cname' ):
-            current_records = self.db_instance.ListRow('records', records_dict)
-            for record in current_records:
-              if( record['record_type'] == 'a' or
-                  record['record_type'] == 'cname' ):
-                raise RecordError('Record already exists with that target '
-                                  'target: %s type: %s' %
-                                  (record['record_type'],
-                                   record['record_target']))
+                          'record_last_user': None}
+
+          if( record['record_type'] == 'cname' ):
+            all_records = self.db_instance.ListRow('records', records_dict)
+            if( len(all_records) > 0 ):
+              raise RecordsBatchError(
+                  'Record already exists with target %s.' % (
+                      record['record_target']))
+          records_dict['record_type'] = u'cname'
+          cname_records = self.db_instance.ListRow('records', records_dict)
+          if( len(cname_records) > 0 ):
+            raise RecordsBatchError('CNAME already exists with target %s.' % (
+                record['record_target']))
+
+          record_args_assignment_dict = self.db_instance.GetEmptyRowDict(
+              'record_arguments_records_assignments')
+          records_dict['record_type'] = record['record_type']
+          raw_records = self.db_instance.ListRow(
+              'records', records_dict, 'record_arguments_records_assignments',
+              record_args_assignment_dict)
+          records_dict['record_last_user'] = self.user_instance.GetUserName()
+          records_dict['record_ttl'] = ttl
+          current_records = (
+              helpers_lib.GetRecordsFromRecordRowsAndArgumentRows(
+              raw_records, record['record_arguments']))
+          for current_record in current_records:
+            for arg in record['record_arguments'].keys():
+              if( arg not in current_record ):
+                break
+              if( record['record_arguments'][arg] is None ):
+                continue
+              if( record['record_arguments'][arg] != current_record[arg] ):
+                break
+            else:
+              raise RecordsBatchError('Duplicate record found')
+
 
           records_dict['record_type'] = record['record_type']
           record_id = self.db_instance.MakeRow('records', records_dict)
-          for k in record['record_arguments'].keys():
+          for arg in record['record_arguments'].keys():
             record_argument_assignments_dict = {
                'record_arguments_records_assignments_record_id': record_id,
                'record_arguments_records_assignments_type': record[
                    'record_type'],
-               'record_arguments_records_assignments_argument_name': k,
-               'argument_value': unicode(record['record_arguments'][k])}
+               'record_arguments_records_assignments_argument_name': arg,
+               'argument_value': unicode(record['record_arguments'][arg])}
             self.db_instance.MakeRow('record_arguments_records_assignments',
                                      record_argument_assignments_dict)
             log_dict['add'].append(record)
@@ -652,18 +625,8 @@ class CoreHelpers(object):
       self.db_instance.EndTransaction()
       success = True
     finally:
-      for operation in log_dict:
-        for record in log_dict[operation]:
-          self.log_instance.LogAction(self.user_instance.user_name,
-                                      u'ProcessRecordsBatch(%s)' % operation,
-                                      u'record_type: %s target: %s '
-                                       'zone_name: %s record_args_dict: %s '
-                                       'view_name: %s' % (
-                                           record['record_type'],
-                                           record['record_target'],
-                                           record['record_zone_name'],
-                                           record['record_arguments'],
-                                           record['view_name']), success)
+      self.log_instance.LogAction(self.user_instance.user_name, function_name,
+                                  current_args, success)
     return row_count
 
 # vi: set ai aw sw=2:
