@@ -33,17 +33,18 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.12'
+__version__ = '0.14'
 
 
-import constants
+import cPickle
 import datetime
-import errors
 import re
 
 import IPy
 
-import table_enumeration
+import constants
+import errors
+import helpers_lib
 
 
 class UnexpectedDataError(errors.DbAccessError):
@@ -248,6 +249,22 @@ class DataValidation(object):
     return False
 
 
+  def isPickleString(self, pickle_string):
+    """Checks that the string can be unpickled.
+
+    Inputs:
+      pickle_string: string to be unpickled.
+
+    Outputs:
+      bool: if it is a valid pickle string
+    """
+    try:
+      cPickle.loads(pickle_string)
+    except (cPickle.PickleError, TypeError):
+      return False
+    return True
+
+
   def ValidateRowDict(self, table_name, row_dict, none_ok=False,
                       all_none_ok=False):
     """Checks row dictionaries for correctness in reference to know data types
@@ -265,7 +282,7 @@ class DataValidation(object):
       UnexpectedDataError: Invalid data type
       UnexpectedDataError: Need to fill out at least one value in dict
     """
-    main_dict = table_enumeration.GetRowDict(table_name)
+    main_dict = helpers_lib.GetRowDict(table_name)
 
     for k in main_dict.iterkeys():
       if( k not in row_dict ):
