@@ -66,7 +66,7 @@ Note: MySQLdb.Error can be raised in almost any function in this module. Please
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.12'
+__version__ = '0.14'
 
 
 import datetime
@@ -74,14 +74,16 @@ import Queue
 import threading
 import time
 import uuid
+import warnings
 
 import MySQLdb
 import MySQLdb.cursors
 
 import constants
 import data_validation
+import embedded_files
 import errors
-import table_enumeration
+import helpers_lib
 
 
 class TransactionError(errors.DbAccessError):
@@ -245,6 +247,15 @@ class dbAccess(object):
         else:
           self.now_serving = None
 
+  def CheckMaintenanceFlag(self):
+    """Checks the maintenance flag in the database.
+
+    Outputs:
+      bool: boolean of maintenance mode
+    """
+    row = self.ListRow('locks', {'lock_name': u'maintenance', 'locked': None})
+    return bool(row[0]['locked'])
+
   def LockDb(self):
     """This function is to lock the whole database for consistent data
     retrevial.
@@ -258,7 +269,7 @@ class dbAccess(object):
                         '`lock_name`="db_lock_lock"')
     time.sleep(self.big_lock_wait)
     self.cursor.execute(
-        'LOCK TABLES %s READ' % ' READ, '.join(constants.TABLES.keys()))
+        'LOCK TABLES %s READ' % ' READ, '.join(self.ListTableNames()))
     self.locked_db = True
 
   def UnlockDb(self):
@@ -292,115 +303,11 @@ class dbAccess(object):
 
     self.data_validation_instance = data_validation.DataValidation(words)
 
-  def GetUserAuthorizationInfo(self, user):
-    """Grabs authorization data from the db and returns a dict.
-
-    This function does two selects on the db, one for forward and one for
-    reverse zones. It also parses the data into a dict for ease of use.
-
-    Inputs:
-      user: string of username
-
-    Raises:
-      UnexpectedDataError: Row did not contain
-                           reverse_range_permissions_access_right or
-                           forward_zone_permissions_access_right
-
-    Outputs:
-      dict: dict with all the relevant information
-        example:
-        {'user_access_level': '2',
-         'user_name': 'shuey',
-         'forward_zones': [
-             {'zone_name': 'cs.university.edu', 'access_right': 'rw'},
-             {'zone_name': 'eas.university.edu', 'access_right': 'r'},
-             {'zone_name': 'bio.university.edu', 'access_right': 'rw'}],
-         'groups': ['cs', 'bio'],
-         'reverse_ranges': [
-             {'cidr_block': '192.168.0.0/24',
-              'access_right': 'rw'},
-             {'cidr_block': '192.168.0.0/24',
-              'access_right': 'r'},
-             {'cidr_block': '192.168.1.0/24',
-              'access_right': 'rw'}]}
-    """
-    auth_info_dict = {}
-    db_data = []
-
-    users_dict = self.GetEmptyRowDict('users')
-    users_dict['user_name'] = user
-    user_group_assignments_dict = self.GetEmptyRowDict('user_group_assignments')
-    forward_zone_permissions_dict = self.GetEmptyRowDict(
-        'forward_zone_permissions')
-    reverse_range_permissions_dict = self.GetEmptyRowDict(
-        'reverse_range_permissions')
-
-    auth_info_dict['user_name'] = user
-    auth_info_dict['groups'] = []
-    auth_info_dict['forward_zones'] = []
-    auth_info_dict['reverse_ranges'] = []
-
-    self.StartTransaction()
-    try:
-      db_data.extend(self.ListRow('users', users_dict, 'user_group_assignments',
-                                  user_group_assignments_dict,
-                                  'forward_zone_permissions',
-                                  forward_zone_permissions_dict))
-
-      db_data.extend(self.ListRow('users', users_dict, 'user_group_assignments',
-                                  user_group_assignments_dict,
-                                  'reverse_range_permissions',
-                                  reverse_range_permissions_dict))
-      if( not db_data ):
-        self.cursor.execute('SELECT access_level FROM users '
-                            'WHERE user_name="%s"' % user)
-        db_data.extend(self.cursor.fetchall())
-
-        if( db_data ):
-          auth_info_dict['user_access_level'] = db_data[0]['access_level']
-          return auth_info_dict
-        else:
-          return {}
-    finally:
-      self.EndTransaction()
-
-    auth_info_dict['user_access_level'] = db_data[0]['access_level']
-    for row in db_data:
-      if( row.has_key('forward_zone_permissions_access_right') ):
-        if( not row['user_group_assignments_group_name'] in
-            auth_info_dict['groups'] ):
-          auth_info_dict['groups'].append(
-              row['user_group_assignments_group_name'])
-
-        if( not {'zone_name': row['forward_zone_permissions_zone_name'],
-                 'access_right': row['forward_zone_permissions_access_right']}
-            in (auth_info_dict['forward_zones']) ):
-          auth_info_dict['forward_zones'].append(
-              {'zone_name': row['forward_zone_permissions_zone_name'],
-               'access_right': row['forward_zone_permissions_access_right']})
-      elif( row.has_key('reverse_range_permissions_access_right') ):
-        if( not row['user_group_assignments_group_name'] in
-            auth_info_dict['groups'] ):
-          auth_info_dict['groups'].append(
-              row['user_group_assignments_group_name'])
-
-        if( not {'cidr_block': row['reverse_range_permissions_cidr_block'],
-                 'access_right': row['reverse_range_permissions_access_right']}
-            in auth_info_dict['reverse_ranges'] ):
-          auth_info_dict['reverse_ranges'].append(
-              {'cidr_block': row['reverse_range_permissions_cidr_block'],
-               'access_right': row['reverse_range_permissions_access_right']})
-      else:
-        raise UnexpectedDataError('Row did not contain '
-                                  'reverse_range_permissions_access_right or '
-                                  'forward_zone_permissions_access_right.')
-    return auth_info_dict
-
   def MakeRow(self, table_name, row_dict):
     """Creates a row in the database using the table name and row dict
     
     Inputs:
-      table_name: string of valid table name from table_enumeration
+      table_name: string of valid table name from constants
       row_dict: dictionary that coresponds to table_name
 
     Raises:
@@ -410,7 +317,7 @@ class dbAccess(object):
     Outputs:
       int: last insert id
     """
-    if( not table_name in table_enumeration.GetValidTables() ):
+    if( not table_name in helpers_lib.GetValidTables() ):
       raise InvalidInputError('Table name not valid: %s' % table_name)
     if( not self.transaction_init ):
       raise TransactionError('Must run StartTansaction before inserting.')
@@ -434,7 +341,7 @@ class dbAccess(object):
     """Counts the amount of records in a table and returns it.
 
     Inputs:
-      table_name: string of valid table name from table_enumeration
+      table_name: string of valid table name from constants
 
     Raises:
       InvalidInputError: Table name not valid
@@ -444,7 +351,7 @@ class dbAccess(object):
       int: number of rows found
     """
 
-    if( not table_name in table_enumeration.GetValidTables() ):
+    if( not table_name in helpers_lib.GetValidTables() ):
       raise InvalidInputError('Table name not valid: %s' % table_name)
     if( not self.transaction_init ):
       raise TransactionError('Must run StartTansaction before getting row '
@@ -457,7 +364,7 @@ class dbAccess(object):
     """Removes a row in the database using the table name and row dict
 
     Inputs:
-      table_name: string of valid table name from table_enumeration
+      table_name: string of valid table name from constants
       row_dict: dictionary that coresponds to table_name
 
     Raises:
@@ -467,7 +374,7 @@ class dbAccess(object):
     Outputs:
       int: number of rows affected
     """
-    if( not table_name in table_enumeration.GetValidTables() ):
+    if( not table_name in helpers_lib.GetValidTables() ):
       raise InvalidInputError('Table name not valid: %s' % table_name)
     if( not self.transaction_init ):
       raise TransactionError('Must run StartTansaction before deleting.')
@@ -487,7 +394,7 @@ class dbAccess(object):
     """Updates a row in the database using search and update dictionaries.
 
     Inputs:
-      table_name: string of valid table name from table_enumeration
+      table_name: string of valid table name from constants
       search_row_dict: dictionary that coresponds to table_name containing
                        search args
       update_row_dict: dictionary that coresponds to table_name containing
@@ -500,7 +407,7 @@ class dbAccess(object):
     Outputs:
       int: number of rows affected
     """
-    if( not table_name in table_enumeration.GetValidTables() ):
+    if( not table_name in helpers_lib.GetValidTables() ):
       raise InvalidInputError('Table name not valid: %s' % table_name)
     if( not self.transaction_init ):
       raise TransactionError('Must run StartTansaction before deleting.')
@@ -537,6 +444,10 @@ class dbAccess(object):
     Inputs:
       args: pairs of string of table name and dict of rows
       kwargs: lock_rows: default False
+              date_column: column to search date range on, if using multiple
+                           tables, date column must be in the first table
+                           in args.
+              date_range: range of dates to search for on date_column
 
       example usage: ListRow('users', user_row_dict,
                              'user_group_assignments', user_assign_row_dict,
@@ -561,14 +472,25 @@ class dbAccess(object):
     if( self.data_validation_instance is None ):
       self.InitDataValidation()
 
-    valid_tables = table_enumeration.GetValidTables()
+    valid_tables = helpers_lib.GetValidTables()
     tables = {}
     table_names = []
     lock_rows = False
+    date_column = None
+    date_range = ()
     if( kwargs ):
       if( 'lock_rows' in kwargs ):
-        lock_rows=kwargs['lock_rows']
+        lock_rows = kwargs['lock_rows']
         del kwargs['lock_rows']
+      if( 'date_column' in kwargs ):
+        date_column = kwargs['date_column']
+        del kwargs['date_column']
+      if( 'date_range' in kwargs ):
+        date_range = kwargs['date_range']
+        del kwargs['date_range']
+      if( bool(date_column) ^ bool(date_range) ):
+        raise InvalidInputError('If date_column or date_range is specified '
+                                'both are needed')
       if( kwargs ):
         raise InvalidInputError('Found unknown option(s): %s' % kwargs.keys())
 
@@ -593,27 +515,39 @@ class dbAccess(object):
                                                       all_none_ok=True)
         tables[current_table_name] = arg
         table_names.append(current_table_name)
-     
+
+    if( date_range ):
+      if( date_column not in args[1] ):
+        raise InvalidInputError('Column %s from date_column not found in row'
+                                'dictionary: %s' % (date_column, args[1]))
+
+      if( constants.TABLES[args[0]][date_column] != 'DateTime' ):
+        raise InvalidInputError('date_column: %s in table %s is not a'
+                                'DateTime type' % (date_column, args[0]))
+      for date in date_range:
+        if( not self.data_validation_instance.isDateTime(date) ):
+          raise InvalidInputError('Date: %s from date_range is not a valid '
+                                  'datetime object' % date)
     query_where = []
     if( len(tables) > 1 ):
       if( not self.foreign_keys ):
-        self.cursor.execute('select table_name, column_name, '
+        self.cursor.execute('SELECT table_name, column_name, '
                             'referenced_table_name, referenced_column_name '
-                            'from information_schema.key_column_usage where '
-                            'referenced_table_name is not null')
+                            'FROM information_schema.key_column_usage WHERE '
+                            'referenced_table_name IS NOT NULL AND '
+                            'referenced_table_schema="%s"' % self.db_name)
         self.foreign_keys = self.cursor.fetchall()
 
       for key in self.foreign_keys:
         if( key['table_name'] in table_names and
             key['referenced_table_name'] in table_names ):
+          
           query_where.append('(%(table_name)s.%(column_name)s='
                              '%(referenced_table_name)s.'
                              '%(referenced_column_name)s)' % key)
-
       if( not query_where ):
         raise InvalidInputError('Multiple tables were passed in but no joins '
                                 'were found')
-
     column_names = []
     search_dict = {}
     for table_name, row_dict in tables.iteritems():
@@ -623,6 +557,12 @@ class dbAccess(object):
           search_dict[k] = v
           query_where.append('%s%s%s%s' % (k, '=%(', k, ')s'))
 
+    if( date_range ):
+      search_dict['begin_date'] = date_range[0]
+      search_dict['end_date'] = date_range[1]
+      query_where.append('%s%s%s%s' % (date_column, '>=%(begin_date)s AND ',
+                                       date_column, '<=%(end_date)s'))
+
     query_end = ''
     if( query_where ):
       query_end = 'WHERE %s' % ' AND '.join(query_where)
@@ -632,6 +572,7 @@ class dbAccess(object):
     query = 'SELECT %s FROM %s %s' % (','.join(column_names),
                                       ','.join(table_names),
                                       query_end)
+
     self.cursor.execute(query, search_dict)
     return self.cursor.fetchall()
 
@@ -640,7 +581,7 @@ class dbAccess(object):
     the given table using the Make/Remove/ListRow functions.
 
     Inputs:
-      table_name: string of valid table name from table_enumeration
+      table_name: string of valid table name from constants
 
     Raises:
       InvalidInputError: Table name not valid
@@ -652,7 +593,7 @@ class dbAccess(object):
          'acl_range_allowed: None,
          'acl_cidr_block': None }
     """
-    row_dict = table_enumeration.GetRowDict(table_name)
+    row_dict = helpers_lib.GetRowDict(table_name)
     if( not row_dict ):
       raise InvalidInputError('Table name not valid: %s' % table_name)
     for k in row_dict.iterkeys():
@@ -667,7 +608,7 @@ class dbAccess(object):
     Outputs:
       list: valid table names
     """
-    table_enumeration.GetValidTables()
+    helpers_lib.GetValidTables()
 
   def GetRecordArgsDict(self, record_type):
     """Get args for a specific record type from the db and shove them into
@@ -769,4 +710,226 @@ class dbAccess(object):
         table_list.append(table_dict[table])
     return table_list
 
+  def GetCurrentTime(self):
+    """Returns datetime object of current time in database.
+
+    Outputs:
+      datetime: current time in the database
+    """
+    self.cursor.execute('SELECT NOW()')
+    return self.cursor.fetchone()['NOW()']
+
+  def CreateRosterDatabase(self, schema=None):
+    """Destroys existing table structure in database and replaces it
+    with schema that is passed in(or default schema).
+
+    DO NOT RUN THIS AGAINST A DATABASE THAT IS NOT READY TO BE CLEARED
+
+    This function is used because of a poorly understood bug in MySQLdb
+    that does not allow our schema to be executed as one big query. The 
+    work around is splitting the whole thing up and commiting each piece
+    separately.
+
+    Inputs:
+      schema: string of sql schema
+    """
+    if( schema is None ):
+      schema = embedded_files.SCHEMA_FILE
+    schema_lines = schema.split('\n')
+    execute_lines = []
+    continued_line = []
+    for line in schema_lines:
+      if( line.lstrip().startswith('#') ):
+        continue
+      if( line.endswith(';') ):
+        continued_line.append(line)
+        execute_lines.append('\n'.join(continued_line))
+        continued_line = []
+      else:
+        continued_line.append(line)
+      
+    warnings.filterwarnings('ignore', 'Unknown table.*')
+    for line in execute_lines:
+      self.StartTransaction()
+      try:
+        self.cursor.execute(line)
+      finally:
+        self.EndTransaction()
+
+  def DumpDatabase(self):
+    """This will dump the entire database to memory.
+
+    This would be done by mysqldump but it needs to be done in the same lock
+    as other processes. So this is a simple mysqldump function.
+
+    Outputs:
+      Dictionary: Dictionary with keys of table name and schema/data for each
+                  table as values.
+    """
+    table_data = {}
+    self.cursor.execute('SHOW TABLES')
+    table_names = self.cursor.fetchall()
+    self.cursor.execute('SET OPTION SQL_QUOTE_SHOW_CREATE=1')
+    for table_name in table_names:
+      table_name = table_name.values()[0]
+      table_data[table_name] = {}
+      self.cursor.execute('SHOW CREATE TABLE %s' % table_name)
+      table_data[table_name]['schema'] = self.cursor.fetchone()['Create Table']
+      self.cursor.execute('DESCRIBE %s' % table_name)
+      table_data[table_name]['columns'] = []
+      table_descriptions = self.cursor.fetchall()
+      for table_description in table_descriptions:
+        table_data[table_name]['columns'].append(table_description['Field'])
+      self.cursor.execute('SELECT %s FROM %s' % 
+                          (','.join(table_data[table_name]['columns']),
+                           table_name))
+      table_rows = self.cursor.fetchall()
+      table_data[table_name]['rows'] = []
+      for row in table_rows:
+        row_dict = {}
+        for k,v in row.iteritems():
+          row_dict[k] = self.connection.literal(v)
+
+        table_data[table_name]['rows'].append(row_dict)
+
+    return table_data
+
+  ### These functions are for the user class
+  def GetUserAuthorizationInfo(self, user):
+    """Grabs authorization data from the db and returns a dict.
+
+    This function does two selects on the db, one for forward and one for
+    reverse zones. It also parses the data into a dict for ease of use.
+
+    Inputs:
+      user: string of username
+
+    Raises:
+      UnexpectedDataError: Row did not contain
+                           reverse_range_permissions_access_right or
+                           forward_zone_permissions_access_right
+
+    Outputs:
+      dict: dict with all the relevant information
+        example:
+        {'user_access_level': '2',
+         'user_name': 'shuey',
+         'forward_zones': [
+             {'zone_name': 'cs.university.edu', 'access_right': 'rw'},
+             {'zone_name': 'eas.university.edu', 'access_right': 'r'},
+             {'zone_name': 'bio.university.edu', 'access_right': 'rw'}],
+         'groups': ['cs', 'bio'],
+         'reverse_ranges': [
+             {'cidr_block': '192.168.0.0/24',
+              'access_right': 'rw'},
+             {'cidr_block': '192.168.0.0/24',
+              'access_right': 'r'},
+             {'cidr_block': '192.168.1.0/24',
+              'access_right': 'rw'}]}
+    """
+    auth_info_dict = {}
+    db_data = []
+
+    users_dict = self.GetEmptyRowDict('users')
+    users_dict['user_name'] = user
+    groups_dict = self.GetEmptyRowDict('groups')
+    user_group_assignments_dict = self.GetEmptyRowDict('user_group_assignments')
+    forward_zone_permissions_dict = self.GetEmptyRowDict(
+        'forward_zone_permissions')
+    reverse_range_permissions_dict = self.GetEmptyRowDict(
+        'reverse_range_permissions')
+
+    auth_info_dict['user_name'] = user
+    auth_info_dict['groups'] = []
+    auth_info_dict['forward_zones'] = []
+    auth_info_dict['reverse_ranges'] = []
+
+    self.StartTransaction()
+    try:
+      db_data.extend(self.ListRow('users', users_dict,
+                                  'groups', groups_dict,
+                                  'user_group_assignments',
+                                  user_group_assignments_dict,
+                                  'forward_zone_permissions',
+                                  forward_zone_permissions_dict))
+
+      db_data.extend(self.ListRow('users', users_dict,
+                                  'groups', groups_dict,
+                                  'user_group_assignments',
+                                  user_group_assignments_dict,
+                                  'reverse_range_permissions',
+                                  reverse_range_permissions_dict))
+      if( not db_data ):
+        self.cursor.execute('SELECT access_level FROM users '
+                            'WHERE user_name="%s"' % user)
+        db_data.extend(self.cursor.fetchall())
+
+        if( db_data ):
+          auth_info_dict['user_access_level'] = db_data[0]['access_level']
+          return auth_info_dict
+        else:
+          return {}
+    finally:
+      self.EndTransaction()
+
+
+    auth_info_dict['user_access_level'] = db_data[0]['access_level']
+    for row in db_data:
+      if( row.has_key('forward_zone_permissions_access_right') ):
+        if( not row['user_group_assignments_group_name'] in
+            auth_info_dict['groups'] ):
+          auth_info_dict['groups'].append(
+              row['user_group_assignments_group_name'])
+
+        if( not {'zone_name': row['forward_zone_permissions_zone_name'],
+                 'access_right': row['forward_zone_permissions_access_right']}
+            in (auth_info_dict['forward_zones']) ):
+          auth_info_dict['forward_zones'].append(
+              {'zone_name': row['forward_zone_permissions_zone_name'],
+               'access_right': row['forward_zone_permissions_access_right']})
+      elif( row.has_key('reverse_range_permissions_access_right') ):
+        if( not row['user_group_assignments_group_name'] in
+            auth_info_dict['groups'] ):
+          auth_info_dict['groups'].append(
+              row['user_group_assignments_group_name'])
+
+        if( not {'cidr_block': row['reverse_range_permissions_cidr_block'],
+                 'access_right': row['reverse_range_permissions_access_right']}
+            in auth_info_dict['reverse_ranges'] ):
+          auth_info_dict['reverse_ranges'].append(
+              {'cidr_block': row['reverse_range_permissions_cidr_block'],
+               'access_right': row['reverse_range_permissions_access_right']})
+      else:
+        raise UnexpectedDataError('Row did not contain '
+                                  'reverse_range_permissions_access_right or '
+                                  'forward_zone_permissions_access_right.')
+    return auth_info_dict
+
+  def GetZoneOrigin(self, zone_name, view_name):
+    """Returns zone origin of zone_name that is passed in.
+    If no zone origin found, return None
+    
+    Inputs:
+      zone_name: string of zone_name
+      view_name: string of view_name
+
+    Outputs:
+      string of zone origin or None
+    """
+    zone_view_assignments_dict = self.GetEmptyRowDict(
+        'zone_view_assignments')
+    zone_view_assignments_dict['zone_view_assignments_zone_name'] =  zone_name
+    zone_view_assignments_dict[
+        'zone_view_assignments_view_dependency'] = view_name
+    
+    zone_view_assignment_rows = self.ListRow(
+        'zone_view_assignments', zone_view_assignments_dict)
+
+    if( zone_view_assignment_rows ):
+      return zone_view_assignment_rows[0]['zone_origin']
+    else:
+      return None
+
+
+
 # vi: set ai aw sw=2:
