@@ -237,7 +237,7 @@ CREATE TABLE `credentials` (
 
   PRIMARY KEY (`credential_id`),
   CONSTRAINT `user_name_4` FOREIGN KEY (`credential_user_name`)
-    REFERENCES `users` (`user_name`) ON UPDATE CASCADE
+    REFERENCES `users` (`user_name`) ON DELETE CASCADE ON UPDATE CASCADE
 
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
@@ -296,7 +296,9 @@ CREATE TABLE `zone_view_assignments` (
     (`zone_view_assignments_view_dependency`) REFERENCES `view_dependencies`
     (`view_dependency`) ON DELETE CASCADE ON UPDATE CASCADE,
   CONSTRAINT `unique_zone_name_zone_dependency` UNIQUE
-    (`zone_view_assignments_zone_name`,`zone_view_assignments_view_dependency`)
+    (`zone_view_assignments_zone_name`,`zone_view_assignments_view_dependency`),
+  CONSTRAINT `unique_zone_origin_view_dependency` UNIQUE
+    (`zone_view_assignments_view_dependency`,`zone_origin`)
 
 ) ENGINE=InnoDB DEFAULT CHARSET=utf8;
 
@@ -649,6 +651,7 @@ CREATE TABLE `audit_log` (
 ##########
 
 INSERT INTO locks (lock_name) VALUES ('db_lock_lock');
+INSERT INTO locks (lock_name) VALUES ('maintenance');
 
 INSERT INTO view_dependencies (view_dependency) VALUES ('any');
 INSERT INTO zone_types (zone_type) VALUES ('master'),('slave'),('forward');
@@ -685,11 +688,12 @@ INSERT INTO record_arguments (record_arguments_type, argument_name,
     ('mx', 'mail_server', '1', 'Hostname'),
     ('ptr', 'assignment_host', '0', 'Hostname');
 
-# code always expects reserved_words to be populated
-INSERT INTO reserved_words (reserved_word) VALUES ('damn');
-
 # make acl any keyword for named.conf
 INSERT INTO acls (acl_name) VALUES ('any');
 INSERT INTO acl_ranges (acl_ranges_acl_name, acl_range_allowed) VALUES 
     ('any', 1);
+
+# Tree exporter needs to audit log with no active user. Here is a user for the
+# exporter.
+INSERT INTO users (user_name, access_level) VALUES ('tree_export_user', 0);
 """
